#include "fs/devpts/FileSystem.h"
#include "fs/devices/DeviceRegistry.h"
#include "fs/devpts/Inode.h"
#include <luna/Alloc.h>
#include <luna/CString.h>
#include <luna/Ignore.h>

Vector<VFS::FileSystem*> g_devpts_instances;

namespace DevPTS
{
    Result<SharedPtr<VFS::FileSystem>> FileSystem::create()
    {
        SharedPtr<FileSystem> fs = TRY(adopt_shared_if_nonnull(new (std::nothrow) FileSystem()));
        SharedPtr<RootInode> root = TRY(make_shared<RootInode>());

        TRY(root->add_entry(root, "."));
        TRY(root->add_entry(root, ".."));

        root->set_self(root, {});
        root->set_fs(*fs, {});
        root->set_inode_number();
        root->m_metadata.mode = 0755;
        root->m_metadata.initialize_times();
        fs->set_root(root);

        TRY(g_devpts_instances.try_append(fs.ptr()));

        return (SharedPtr<VFS::FileSystem>)fs;
    }

    Result<u64> FileSystem::allocate_inode_number()
    {
        return err(ENOTSUP);
    }

    FileSystem::FileSystem()
    {
        m_host_device_id = DeviceRegistry::next_null_device_id();
    }

    Result<void> FileSystem::set_mount_dir(SharedPtr<VFS::Inode> parent)
    {
        return m_root_inode->replace_entry(parent, "..");
    }

    Result<void> FileSystem::reset_mount_dir()
    {
        return m_root_inode->replace_entry(m_root_inode, "..");
    }

    void FileSystem::set_root(SharedPtr<VFS::Inode> root)
    {
        m_root_inode = root;
    }

    FileSystem::~FileSystem()
    {
        g_devpts_instances.remove_first_matching([this](VFS::FileSystem* ptr) { return ptr == this; });
    }
}
